
#include "BlockingCommPort.h"

#include <unistd.h>
#include <fcntl.h> 


/**
 *  The constructor creates an object for the given communication device.
 *
 *  @param commPortname name of the communication device (e. g.: /dev/ttyACM0)
 */
BlockingCommPort::BlockingCommPort(const char* commPortname) {

    this->commPortname = commPortname;
    fd = -1;
    lastError = 0;
}


/**
 *  The destructor shuts down the device if it is still open.
 */
BlockingCommPort::~BlockingCommPort() {

    close();
}



/**
 *  The function opens the device.
 *
 *  The device to open has been specified in the constructor call.
 *  The device is configured immediately after the open succeeds.
 *  If the device already open when this function is called then
 *  only reconfiguration is done.
 *
 *  If the function call fails then the {@link #lastError} is set
 *  and can be retrieved by calling {@link #getLastError()}.
 *
 *  @param baudrate one of the baudrate constants (e. g. B9600, B57600, ...)
 *  @return true on success, false on failure
 */
bool BlockingCommPort::open(speed_t baudrate) {

    /* The device is opened if the file descriptor does not indicate that the device
       is opened but if the device is already opened then the device is not opened
       but only recofigured. */
    if (fd < 0) {
    
        /*
         *  Options for opening the device:
         *  O_RDWR: opens the device for reading and for writing
         *  O_NOCTTY: the device will not become the controlling terminal of the process
         */
        fd = ::open (commPortname, O_RDWR | O_NOCTTY);
    }

    if (fd >= 0) {

        if (!configureDevice(baudrate)) {

            close();
        }
    }
    else {

        lastError = errno;
    }
    
    return (fd >= 0);
}


/**
 *  The function closes the device if it is open.
 */
void BlockingCommPort::close() {

    /* fd is only a valid file descriptor if it is >= 0, otherwise the device is closed */
    if (fd >= 0) {

        ::close(fd);
    }
    fd = -1;
    lastError = 0;
}


/**
 *  The function configures the device.
 *
 *  The device must be open. fd must be valid.
 *
 *  If the function call fails then the {@link #lastError} is set
 *  and can be retrieved by calling {@link #getLastError()}.
 *
 *  @param baudrate one of the baudrate constants (e. g. B9600, B57600, ...)
 *  @return true on success, false on failure
 */
bool BlockingCommPort::configureDevice(speed_t baudrate) {

    bool result = true;
    
    struct termios config;
    if (tcgetattr (fd, &config) == 0) {

        /* Control mode flags: 8 Bit, no parity */
        config.c_cflag &= ~(CSIZE | PARENB | CSTOPB);
        config.c_cflag |= CS8 | CLOCAL | CREAD;

        /* Local mode flags: no echo, no signalling, raw input */
        config.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG);

        /* Input mode flags; disable software flow control */
        config.c_iflag &= ~(IXON | IXOFF | IXANY);

        /* Output mode flags: no post processing, raw output */
        config.c_oflag &= ~OPOST;

        /* read call reads any number of characters */
        config.c_cc[VMIN] = 0;

        /* blocking timeout in 10th of seconds */
        config.c_cc[VTIME] = 5;

        /* set serial communication baud rate */
        cfsetspeed(&config, B9600);

        if (tcsetattr (fd, TCSAFLUSH, &config) != 0) {
            
            lastError = errno;
            result = false;
        }
    }
    else {
        
        lastError = errno;
        result = false;
    }
    return result;
}


/**
 *  The function writes characters to the device.
 *
 *  If the function call fails then the {@link #lastError} is set
 *  and can be retrieved by calling {@link #getLastError()}.
 *
 *  @param buffer the buffer that stores characters to write to the device
 *  @param bytesToWrite number of valid bytes in buffer, number of bytes to write
 *  @param [out] bytesWritten number of bytes actually written
 *  @return true on success, false on failure
 */
bool BlockingCommPort::writeBuffer(unsigned char* buffer, int bytesToWrite, int& bytesWritten) {

    bytesWritten = write(fd, buffer, bytesToWrite);
    if (bytesWritten < 1) {
            
        lastError = errno;
    }

    return bytesWritten >= 0;
}


/**
 *  The function reads characters from the device.
 *
 *  If the function call fails then the  {@link #lastError} is set
 *  and can be retrieved by calling {@link #getLastError()}. A
 *  failure is indicted by returning false. On success the
 *  characters are returned in the buffer.
 *  If no character is read due to timeout then the
 *  function returns true and sets the out parameter
 *  bytesToRead to 0.
 *
 *  @param buffer the buffer that retrieves characters received from the device
 *  @param bytesToRead number of bytes to read
 *  @param [out] bytesRead number of bytes actually read
 *  @return true on success, false on failure
 */
bool BlockingCommPort::readBuffer(unsigned char* buffer, int bytesToRead, int& bytesRead) {

    bool result = false;

    bytesRead = read(fd, buffer, bytesToRead);
    if (bytesRead < 1) {
            
        lastError = errno;
    }
    else {

        result = true;
    }
    return result;
}
