
#ifndef BLOCKING_COMM_PORT
#define BLOCKING_COMM_PORT

/*
 *  BlockingCommPort class
 *
 *  The file implements the class BlockingCommPort.
 *  It provides configuration and blocking read/write (with timeout)
 *  for serial devices. The class is intended to support
 *  microcontroller debugging. It is not useful for big data
 *  transfers.
 *
 *  This is example code only. It is provided with no warranty.
 */

#include <errno.h>
#include <termios.h>

/**
 *  Class for controlling an UART device on Linux
 *
 *  The class provides an interface class for any serial port.
 *  After creating an object the serial port must be opened
 *  by using the function {@link #open(speed_t baudrate)}.
 *  Then the caller can use the functions {@link #readBuffer}
 *  and {@link #writeBuffer} to receive or send data.
 *  The serial port can be closed and reopened mutiple times
 *  by the object.
 *
 *  The port is opened with 1 stop bit and no parity. Only
 *  the speed is configurable.
 *
 *  @author michael.schatz@ib-schatz.de
 */

class BlockingCommPort {

public:
    BlockingCommPort(const char* commPortname);
    virtual ~BlockingCommPort();

    virtual bool open(speed_t baudrate);
    virtual void close();

    virtual bool writeBuffer(unsigned char* buffer, int bytesToWrite, int& bytesWritten);
    virtual bool readBuffer(unsigned char* buffer, int bytesToRead, int& bytesRead);

    /**
     *  The function returns the last known failure code.
     *
     *  If any function indicates the failure then this function
     *  returns the errno failure code. It can be printed with
     *  the OS function strerror.
     *
     *  @return the last known failure code
     */
    virtual error_t getLastError() { return lastError; }
    
private:

    /** the name of the communication device (e. g.: /dev/ttyACM0) */
    const char *commPortname;

    /** the file descriptor of the device, < 0 if the device is not opened */
    int fd;

    /** the last known error */
    error_t lastError;
    
    bool configureDevice(speed_t baudrate);
};


#endif
